Sub PrintDebug(debugStr$ As String)
  if type(m) <> "roAssociativeArray" then stop
  m.log.SendLine(debugStr$)
End Sub

REM TODO:
REM   Network configuration from file (Ethernet and Wifi)
REM   Portrait configuration


Sub Main()
  xpress = CreateObject("roAssociativeArray")
  xpress.log = CreateObject("roSystemLog") 
  xpress.PrintDebug = PrintDebug
  xpress.PrintDebug("Start DISE Xpress")

  xpress.PrintDebug("Loading XML config")
  config=CreateObject("roXMLElement")
  if not config.Parse(ReadAsciiFile("brightsign_config.xml")) then
    config = invalid
  end if
  
'registrySection = CreateObject ("roRegistrySection", "networking")
'registrySection.Write ("http_server","80")
'registrySection.Flush()  
  
  videoMode = CreateObject("roVideoMode")
  resX = videoMode.GetResX()
  resY = videoMode.GetResY()
  xpress.PrintDebug("Ignoring ImageSizeThreshold")
  videoMode.SetImageSizeThreshold({ignore: 1})
  videoMode = invalid

  xpress.PrintDebug("Width" + strI(resX))
  xpress.PrintDebug("Height" + strI(resY))
  
  xpress.PrintDebug("Create roHtmlWidget")
  msgPort = CreateObject("roMessagePort")
  r = CreateObject("roRectangle", 0, 0, resX, resY)
  h = CreateObject("roHtmlWidget", r)
  h.SetPort(msgPort)
  ok = h.SetHWZDefault("on")
  ok = h.AllowJavaScriptUrls({ all: "*" })
  ok = h.StartInspectorServer(2999)
  
  'Portrait mode  
  if config<>invalid and config.portrait<>invalid and config.portrait.gettext()="true" then
    xpress.PrintDebug("Portrait mode")
    ok = h.SetTransform("rot90")
  else
    xpress.PrintDebug("Landscape mode")
  end if
  ok = h.SetURL("file:///index.html")

  xpress.PrintDebug("Setup storage")
  ' convert from percentage settings to absolute settings
  du = CreateObject("roStorageInfo", "./")
  totalCardSizeMB% = du.GetSizeInMegabytes()

  xpress.PrintDebug("totalCardSizeMB" + strI(totalCardSizeMB%))
  
  ' pool size for published data
  publishedDataSizeLimitPercentage% = 5
  maximumPublishedDataPoolSizeMB% = publishedDataSizeLimitPercentage% * totalCardSizeMB% / 100
  xpress.PrintDebug("maximumPublishedDataPoolSizeMB" + strI(maximumPublishedDataPoolSizeMB%))

  ' pool size for dynamic data
  dynamicDataSizeLimitPercentage% = 5
  maximumDynamicDataPoolSizeMB% = dynamicDataSizeLimitPercentage% * totalCardSizeMB% / 100
  xpress.PrintDebug("maximumDynamicDataPoolSizeMB" + strI(maximumDynamicDataPoolSizeMB%))

  ' size for html data
  htmlDataSizeLimitPercentage% = 5
  maximumHTMLDataPoolSizeMB% = htmlDataSizeLimitPercentage% * totalCardSizeMB% / 100
  if maximumHTMLDataPoolSizeMB% >= 2048 then
    maximumHTMLDataPoolSizeMB% = 2047
  endif  
  xpress.PrintDebug("maximumHTMLDataPoolSizeMB" + strI(maximumHTMLDataPoolSizeMB%) + " " + strI(maximumHTMLDataPoolSizeMB% * 1024 * 1024))

  ' size for html local storage
  htmlLocalStorageSizeLimitPercentage% = 60
  maximumHTMLLocalStoragePoolSizeMB% = htmlLocalStorageSizeLimitPercentage% * totalCardSizeMB% / 100
  if maximumHTMLLocalStoragePoolSizeMB% >= 2048 then
    maximumHTMLLocalStoragePoolSizeMB% = 2047
  endif
  xpress.PrintDebug("maximumHTMLLocalStoragePoolSizeMB" + strI(maximumHTMLLocalStoragePoolSizeMB%) + " " + strI(maximumHTMLLocalStoragePoolSizeMB% * 1024 * 1024))

  ' size for html indexed db
  htmlIndexedDBSizeLimitPercentage% = 25
  maximumHTMLIndexedDBPoolSizeMB% = htmlIndexedDBSizeLimitPercentage% * totalCardSizeMB% / 100
  if maximumHTMLIndexedDBPoolSizeMB% >= 2048 then
    maximumHTMLIndexedDBPoolSizeMB% = 2047
  endif
  
  xpress.PrintDebug("maximumHTMLIndexedDBPoolSizeMB" + strI(maximumHTMLIndexedDBPoolSizeMB%) + " " + strI(maximumHTMLIndexedDBPoolSizeMB% * 1024 * 1024))

  ok = CreateDirectory("pool")
  ok = CreateDirectory("feedPool")
  assetPool = CreateObject("roAssetPool", "pool")
  feedPool = CreateObject("roAssetPool", "feedPool")  
  
  assetPool.SetMaximumPoolSizeMegabytes( maximumPublishedDataPoolSizeMB% )
  feedPool.SetMaximumPoolSizeMegabytes( maximumDynamicDataPoolSizeMB% )
  ok = h.SetAppCacheSize( maximumHTMLDataPoolSizeMB% * 1024.0 * 1024.0 )
  ok = h.SetAppCacheDir( "AppCache" )
  ok = h.SetLocalStorageDir("localstorage")
  ok = h.SetLocalStorageQuota(maximumHTMLLocalStoragePoolSizeMB% * 1024 * 1024)
  ok = h.SetWebDatabaseDir("IndexedDB")
  ok = h.SetWebDatabaseQuota(maximumHTMLIndexedDBPoolSizeMB% * 1024 * 1024)
  
  xpress.PrintDebug("Get network configuraion")
  nc = CreateObject("roNetworkConfiguration", 0)
  networkConfig = nc.GetCurrentConfig()
  nc = invalid
  
  audioAnalog = CreateObject("roAudioOutput", "Analog")
  audioHDMI = CreateObject("roAudioOutput", "HDMI")
  
  xpress.PrintDebug("Delay 5 seconds before start")
  sleep(5000)
  
  xpress.PrintDebug("Show page")
  h.Show()
  
  while true
    xpress.PrintDebug("Wait for data")
    ev = wait(0, msgPort)
    'ev = msgPort.WaitMessage(0)
    if type(ev) <> "roHtmlWidgetEvent" then
        xpress.PrintDebug("<>roHtmlWidgetEvent")
        xpress.PrintDebug("type(ev)" + type(ev))
    else
      payload = ev.GetData()
      xpress.PrintDebug("payload")
      xpress.PrintDebug("Reason: " + payload.reason)
      if type(payload) = "roAssociativeArray" and payload.reason = "message" then
          messageType = payload.message.Lookup("type")
          messageData = payload.message.Lookup("data")
          if messageType = invalid or messageData = invalid then
              xpress.PrintDebug("Incorrect message")
          else
              xpress.PrintDebug("Message type: " +messageType)
              if messageType = "status" then
                xpress.PrintDebug("Message data: " + messageData)
                xpress.PrintDebug("Send test message2")
                h.PostJSMessage({ type: "network", name: "ip", data: networkConfig.ip4_address })
                h.PostJSMessage({ type: "audio", name: "mute", data: False })
                h.PostJSMessage({ type: "audio", name: "volume", data: 100% })
              else if messageType = "action" then
                messageValue = payload.message.Lookup("value")
                if messageData = "reboot" then
                  xpress.PrintDebug("Rebooting device")
                  RebootSystem()
                  stop
                else if messageData = "mute" then
                  if (messageValue = invalid) then
                    xpress.PrintDebug("Mute: Incorrect value")
                  else
                    value = True
                    if (messageValue = "0")
                      value = False
                    end if
                    xpress.PrintDebug("Mute: Analog audio " + messageValue)
                    audioAnalog.SetMute(value)
                    xpress.PrintDebug("Mute: HDMI audio " + messageValue)
                    audioHDMI.SetMute(value)
                    h.PostJSMessage({ type: "audio", name: "mute", data: value })
                  end if
                else if messageData = "volume" then
                  if (messageValue = invalid) then
                    xpress.PrintDebug("Volume: Incorrect value")
                  else
                    xpress.PrintDebug("Volume: Analog audio " + messageValue)
                    audioAnalog.SetVolume(messageValue)
                    xpress.PrintDebug("Volume: HDMI audio " + messageValue)
                    audioHDMI.SetVolume(messageValue)
                    h.PostJSMessage({ type: "audio", name: "volume", data: messageValue })
                  end if 
                end if 
              end if
          end if
      end if
    end if
  end while  
  xpress.PrintDebug("Should not get here!")  
End Sub

